'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _extends = require('babel-runtime/helpers/extends')['default'];

var _slicedToArray = require('babel-runtime/helpers/sliced-to-array')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.idFetcher = idFetcher;
exports.isConnection = isConnection;
exports.handleConnection = handleConnection;
exports.sequelizeNodeInterface = sequelizeNodeInterface;
exports.nodeAST = nodeAST;
exports.nodeType = nodeType;
exports.sequelizeConnection = sequelizeConnection;

var _graphqlRelay = require('graphql-relay');

var _graphql = require('graphql');

var _base64Js = require('./base64.js');

var _resolver = require('./resolver');

var _resolver2 = _interopRequireDefault(_resolver);

var NodeTypeMapper = (function () {
  function NodeTypeMapper(sequelize) {
    var _this = this;

    _classCallCheck(this, NodeTypeMapper);

    this.models = _Object$keys(sequelize.models);
    this.models.forEach(function (model) {
      _this[model] = null;
    });
  }

  _createClass(NodeTypeMapper, [{
    key: 'mapTypes',
    value: function mapTypes(types) {
      var _this2 = this;

      _Object$keys(types).forEach(function (type) {
        _this2[type] = types[type];
      });
    }
  }]);

  return NodeTypeMapper;
})();

function idFetcher(sequelize, nodeTypeMapper) {
  return function (globalId) {
    var _fromGlobalId = (0, _graphqlRelay.fromGlobalId)(globalId);

    var type = _fromGlobalId.type;
    var id = _fromGlobalId.id;

    var models = _Object$keys(sequelize.models);
    if (models.some(function (model) {
      return model === type;
    })) {
      return sequelize.models[type].findById(id);
    }
    if (nodeTypeMapper[type]) {
      return nodeTypeMapper[type];
    }
    return null;
  };
}

function isConnection(type) {
  return typeof type.name !== 'undefined' && type.name.endsWith('Connection');
}

function handleConnection(values, args) {
  return (0, _graphqlRelay.connectionFromArray)(values, args);
}

function sequelizeNodeInterface(sequelize) {
  var nodeTypeMapper = new NodeTypeMapper(sequelize);
  var nodeObjects = (0, _graphqlRelay.nodeDefinitions)(idFetcher(sequelize, nodeTypeMapper), function (obj) {
    var name = obj.Model ? obj.Model.options.name.singular : obj.name;
    return nodeTypeMapper[name];
  });
  return _extends({
    nodeTypeMapper: nodeTypeMapper
  }, nodeObjects);
}

function nodeAST(connectionAST) {
  return connectionAST.fields.edges.fields.node;
}

function nodeType(connectionType) {
  return connectionType._fields.edges.type.ofType._fields.node.type;
}

function sequelizeConnection(_ref) {
  var name = _ref.name;
  var nodeType = _ref.nodeType;
  var target = _ref.target;
  var orderBy = _ref.orderBy;

  var _connectionDefinitions = (0, _graphqlRelay.connectionDefinitions)({ name: name, nodeType: nodeType });

  var edgeType = _connectionDefinitions.edgeType;
  var connectionType = _connectionDefinitions.connectionType;

  var model = target.target ? target.target : target;

  var SEPERATOR = '$';
  var PREFIX = 'arrayconnection' + SEPERATOR;

  if (orderBy === undefined) {
    orderBy = new _graphql.GraphQLList(new _graphql.GraphQLEnumType({
      name: name + 'ConnectionOrder',
      values: {
        ID: [model.primaryKeyAttribute, 'ASC']
      }
    }));
  }

  var $connectionArgs = _extends({}, _graphqlRelay.connectionArgs, {
    orderBy: {
      type: new _graphql.GraphQLList(orderBy)
    }
  });

  var _resolve = (0, _resolver2['default'])(target, {
    handleConnection: false,
    before: function before(options, args) {
      if (args.first || args.last) {
        options.limit = parseInt(args.first || args.last, 10);
      }

      if (!args.orderBy) {
        args.orderBy = orderBy.values[0].value;
      }

      var orderAttribute = orderByAttribute(args.orderBy);
      var orderDirection = args.orderBy[0][1];

      options.order = args.orderBy;
      options.attributes.push(orderAttribute);

      if (args.after || args.before) {
        var after = fromCursor(args.after || args.before);
        var orderValue = after.orderValue;

        if (model.rawAttributes[orderAttribute].type instanceof model.sequelize.constructor.DATE) {
          orderValue = new Date(orderValue);
        }

        options.where = options.where || {};

        if (orderDirection === 'ASC' || args.before) {
          options.where[orderByAttribute(args.orderBy)] = {
            $gt: orderValue
          };
        } else {
          options.where[orderByAttribute(args.orderBy)] = {
            $lt: orderValue
          };
        }
      }

      return options;
    }
  });

  var orderByAttribute = function orderByAttribute(orderBy) {
    return orderBy[0][0];
  };

  var toCursor = function toCursor(value, orderBy) {
    var id = value.get(model.primaryKeyAttribute);
    var orderValue = value.get(orderByAttribute(orderBy));
    return (0, _base64Js.base64)(PREFIX + id + SEPERATOR + orderValue);
  };

  var fromCursor = function fromCursor(cursor) {
    cursor = (0, _base64Js.unbase64)(cursor);
    cursor = cursor.substring(PREFIX.length, cursor.length);

    var _cursor$split = cursor.split(SEPERATOR);

    var _cursor$split2 = _slicedToArray(_cursor$split, 2);

    var id = _cursor$split2[0];
    var orderValue = _cursor$split2[1];

    return {
      id: id,
      orderValue: orderValue
    };
  };

  return {
    connectionType: connectionType,
    edgeType: edgeType,
    nodeType: nodeType,
    connectionArgs: $connectionArgs,
    resolve: function resolve(source, args, info) {
      return _resolve(source, args, info).then(function (values) {
        var edges = values.map(function (value) {
          return {
            cursor: toCursor(value, args.orderBy),
            node: value
          };
        });

        var firstEdge = edges[0];
        var lastEdge = edges[edges.length - 1];

        return {
          edges: edges,
          pageInfo: {
            startCursor: firstEdge ? firstEdge.cursor : null,
            endCursor: lastEdge ? lastEdge.cursor : null,
            hasPreviousPage: null,
            hasNextPage: null
          }
        };
      });
    }
  };
}